const axios = require("axios");
const githubUsernameRegex = require("github-username-regex");
const moment = require("moment");
const retryer = require("../common/retryer");
const calculateRank = require("../common/calculateRank");
const { request, logger, CustomError } = require("../common/utils");

require("dotenv").config();

const fetcher = (variables, token) => {
  return request(
    {
      query: `
      query userInfo($login: String!) {
        user(login: $login) {
          name
          login
          avatarUrl
          bio
          company
          createdAt
          isGitHubStar
          location
          websiteUrl
          isHireable
          organizations {
            totalCount
          }
          contributionsCollection {
            totalCommitContributions
            restrictedContributionsCount
          }
          repositoriesContributedTo(first: 100, contributionTypes: [COMMIT, ISSUE, PULL_REQUEST, REPOSITORY], orderBy: {direction: DESC, field: STARGAZERS}) {
            totalCount
            nodes {
              nameWithOwner
              stargazers {
                totalCount
              }
            }
          }
          pullRequests(first: 1) {
            totalCount
          }
          issues(first: 1) {
            totalCount
          }
          followers {
            totalCount
          }
          answeredDiscussions:repositoryDiscussionComments(onlyAnswers:true){
            totalCount
          }
          totalDiscussionComments:repositoryDiscussionComments{
            totalCount
          }
          repositories(first: 100, ownerAffiliations: OWNER, orderBy: {direction: DESC, field: STARGAZERS}) {
            totalCount
            nodes {
              nameWithOwner
              stargazers {
                totalCount
              }
            }
          }
        }
      }
      `,
      variables,
    },
    {
      Authorization: `bearer ${token}`,
    },
  );
};

const fetchUserBio=async(username)=>{
  if(!githubUsernameRegex.test(username)){
    logger.log("invalid username");
    return 0;
  }
  const getUserBio= (variables,token)=>{
    return axios({
      method:"get",
      url:`https://api.github.com/users/${username}`,
      headers: {
        "Content-Type": "application/json",
        Accept: "application/vnd.github.cloak-preview",
        Authorization: `bearer ${token}`,
      },
    })
  }

  try {
    let res = await retryer(getUserBio, { login: username });
    if (res.data) {
      return {name:res.data.name,bio:res.data.bio,avatar:res.data.avatar_url || null};
    }
  } catch (err) {
    logger.log(err);
    // just return 0 if there is something wrong so that
    // we don't break the whole app
    return 0;
  }
}
const checkUserValidity=async(username)=>{
  if(!githubUsernameRegex.test(username)){
    logger.log("invalid username");
    return 0;
  }
  const getUsername= (variables,token)=>{
    return axios({
      method:"get",
      url:`https://api.github.com/users/${username}`,
      headers: {
        "Content-Type": "application/json",
        Accept: "application/vnd.github.cloak-preview",
        Authorization: `bearer ${token}`,
      },
    })
  }

  try {
    let res = await retryer(getUsername, { login: username });
    if (res && res.data) {
      if(res.data.name){

        return {msg:res.data.name,valid:true};
      }else{
        return {msg:username,valid:true};
      }
    }else{
      return{msg:'not exists',valid:false};
    }
  } catch (err) {
    logger.log(err);
    // just return 0 if there is something wrong so that
    // we don't break the whole app
    return {valid:false};
  }
}


const totalCommitsFetcher = async (username) => {
  if (!githubUsernameRegex.test(username)) {
    logger.log("Invalid username");
    return 0;
  }

 
  const fetchTotalCommits = (variables, token) => {
    return axios({
      method: "get",
      url: `https://api.github.com/search/commits?q=author:${variables.login}`,
      headers: {
        "Content-Type": "application/json",
        Accept: "application/vnd.github.cloak-preview",
        Authorization: `bearer ${token}`,
      },
    });
  };

  try {
    let res = await retryer(fetchTotalCommits, { login: username });
    if (res.data.total_count) {
      return res.data.total_count;
    }
  } catch (err) {
    logger.log(err);
    // just return 0 if there is something wrong so that
    // we don't break the whole app
    return 0;
  }
};

async function fetchStats(
  username,
  count_private = true,
  include_all_commits = true,
) {
  if (!username) throw Error("Invalid username");

  const stats = {
    name: "",
    totalPRs: 0,
    totalCommits: 0,
    totalIssues: 0,
    totalStars: 0,
    contributedTo: 0,
    followers:0,
    totalRepositories:0,
    rank: { level: 0, score: 0, badge:"newbie",points:0 },
  };

  let res = await retryer(fetcher, { login: username });

  if (res.data.errors) {
    logger.error(res.data.errors);
    throw new CustomError(
      res.data.errors[0].message || "Could not fetch user",
      CustomError.USER_NOT_FOUND,
    );
  }

  const user = res.data.data.user;
  console.log('whole list',res.data)

  stats.name = user.name || user.login;
  stats.totalIssues = user.issues.totalCount;

  // normal commits
  stats.totalCommits = user.contributionsCollection.totalCommitContributions;

  // if include_all_commits then just get that,
  // since totalCommitsFetcher already sends totalCommits no need to +=
  if (include_all_commits) {
    stats.totalCommits = await totalCommitsFetcher(username);
  }

  // if count_private then add private commits to totalCommits so far.
  if (count_private) {
    stats.totalCommits +=
      user.contributionsCollection.restrictedContributionsCount;
  }

  stats.totalPRs = user.pullRequests.totalCount;
  stats.contributedTo = user.repositoriesContributedTo.totalCount;
  stats.totalRepositories = user.repositories.totalCount;
  stats.followers = user.followers.totalCount;
  stats.isHireable = user.isHireable; 
  stats.totalDiscussions = user.totalDiscussionComments.totalCount;
  stats.answeredDiscussions = user.answeredDiscussions.totalCount;

  // calculate total stargazers count
  stats.totalStars = user.repositories.nodes.reduce((prev, curr) => {
    return prev + curr.stargazers.totalCount;
  }, 0);

  // calculate user account age
  const todayDate=new moment(Date.now());
  const joinedDate=new moment(user.createdAt);
  stats.profileAge=Math.round(moment.duration(todayDate.diff(joinedDate)).asYears()*10)/10;

  // filter contributed repos according to the number of stargazers
  stats.highRepos=[];
  stats.lowRepos=[];
  stats.midRepos=[];
   user.repositoriesContributedTo.nodes.map(repo=>{
    if(repo.stargazers.totalCount>=10000){
      stats.highRepos.push(repo.nameWithOwner)
    }else if(repo.stargazers.totalCount<10000 && repo.stargazers.totalCount>=5000){
      stats.midRepos.push(repo.nameWithOwner)
    } else if(repo.stargazers.totalCount<5000 && repo.stargazers.totalCount>=500){
      stats.lowRepos.push(repo.nameWithOwner)
    }
  })

  // filter individual repos according to the number of stargazers
  stats.highIndividualRepos=[];
  stats.midIndividualRepos=[];
  stats.lowIndividualRepos=[];
  let rounds =5;
  let repoCount =0;
  user.repositories.nodes.map(repo=>{
    if(repo.stargazers.totalCount>1000){
      repoCount++;
      stats.highIndividualRepos.push(repo.nameWithOwner)
    }else if(repo.stargazers.totalCount>100){
      repoCount++;
      stats.midIndividualRepos.push(repo.nameWithOwner)
    }else if(0<rounds && repoCount<10){
      // push only the first 5 (see round variable) repos with highest stargazers
      rounds--;
      stats.lowIndividualRepos.push(repo.nameWithOwner)
    }
  })

  stats.rank = calculateRank({
    totalCommits: stats.totalCommits,
    totalRepos: user.repositories.totalCount,
    followers: user.followers.totalCount,
    contributions: stats.contributedTo,
    stargazers: stats.totalStars,
    totalPRs: stats.totalPRs,
    issues: stats.totalIssues,
    createdAt:user.createdAt
  });
  const userBio={
    name:user.name,
    username:user.login,
    avatar_url:user.avatarUrl,
    bio:user.bio,
    company:user.company,
    created_at:user.createdAt,
    location:user.location,
    website:user.website,
  }
  return {stats,userBio};
}


exports.fetchStats=fetchStats;
exports.fetchUserBio=fetchUserBio;
exports.checkUserValidity=checkUserValidity;